@@ -4,6 +4,7 @@ lightweight wrapper around Redis.
 
 """
 import datetime
+import json
 import redis
 
 from django.conf import settings
@@ -21,6 +22,8 @@ class R(object):
 
         Valid keyword arguments:
 
+        * ``categories_key`` -- The key storing a set of all metric Categories
+          (default is "categories")
         * ``metric_slugs_key`` -- The key storing a set of all metrics slugs
           (default is "metric-slugs")
         * ``gauge_slugs_key`` -- The key storing a set of all slugs for gauges
@@ -30,6 +33,7 @@ class R(object):
         * ``db``   -- Redis DB (default settings.REDIS_METRICS_DB)
 
         """
+        self._categories_key = kwargs.get('categories_key', 'categories')
         self._metric_slugs_key = kwargs.get('metric_slugs_key', 'metric-slugs')
         self._gauge_slugs_key = kwargs.get('gauge_slugs_key', 'gauge-slugs')
 
@@ -52,14 +56,14 @@ class R(object):
         self.r = redis.StrictRedis(host=self.host, port=self.port, db=self.db)
 
     def _date_range(self, since=None):
-        """Returns a generator that yields ``datetime.date`` objects from the
-        ``since`` date until *now*. If ``since`` is omitted, returns dates for
-        one year.
+        """Returns a generator that yields ``datetime.datetime`` objects from
+        the ``since`` date until *now*. If ``since`` is omitted, returns dates
+        for one year.
 
-        * ``since`` -- a ``datetime.date`` object or None.
+        * ``since`` -- a ``datetime.datetime`` object or None.
 
         """
-        now = datetime.date.today()
+        now = datetime.datetime.today()
         if since is None:
             since_days = 365  # assume 1 year :-/
         else:
@@ -67,6 +71,38 @@ class R(object):
             since_days = delta.days + 1
         return (now - datetime.timedelta(days=d) for d in range(since_days))
 
+    def _category_key(self, category):
+        return u"c:{0}".format(category)
+
+    def _category_slugs(self, category):
+        """Returns a list of the metric slugs for the given category"""
+        key = self._category_key(category)
+        slugs = self.r.get(key)
+        return [] if slugs is None else json.loads(slugs)
+
+    def _categorize(self, slug, category):
+        """Add the ``slug`` to the ``category``. We store category data as
+        JSON strings, with a key of the form::
+
+            c:<category name>
+
+        The data is a JSON-serialized list of metric slugs::
+
+            '["slug-a", "slug-b", ...]'
+
+        """
+        # Store existing slugs in a set, so we don't have duplicates
+        metric_slugs = set(self._category_slugs(category))
+        metric_slugs.add(slug)
+
+        # sets are not JSON-serializable, so convert to a list before storing
+        json_data = json.dumps(list(metric_slugs))
+        key = self._category_key(category)
+        self.r.set(key, json_data)
+
+        # Store all category names in a Redis set, for easy retrieval
+        self.r.sadd(self._categories_key, category)
+
     def _build_keys(self, slug, date=None, granularity='all'):
         """Builds redis keys used to store metrics.
 
@@ -105,7 +141,29 @@ class R(object):
         keys = self.r.smembers(self._metric_slugs_key)
         return set(s.split(":")[1] for s in keys)
 
-    def metric(self, slug, num=1):
+    def metric_slugs_by_category(self):
+        """Return a dictionary of category->metrics data:
+
+            {<category_name>: [<slug1>, <slug2>, ...]}
+
+        """
+        result = {}
+        categories = self.r.smembers(self._categories_key)
+        for category in categories:
+            result[category] = self._category_slugs(category)
+
+        # We also need to see the uncategorized metric slugs, so need some way
+        # to check which slugs are not already stored.
+        categorized_metrics = set([  # Flatten the list of metrics
+            slug for sublist in result.values() for slug in sublist
+        ])
+        f = lambda slug: slug not in categorized_metrics
+        uncategorized = filter(f, self.metric_slugs())
+        if len(uncategorized) > 0:
+            result['Uncategorized'] = uncategorized
+        return result
+
+    def metric(self, slug, num=1, category=None):
         """Records a metric, creating it if it doesn't exist or incrementing it
         if it does. All metrics are prefixed with 'm', and automatically
         aggregate for Day, Week, Month, and Year.
@@ -131,6 +189,9 @@ class R(object):
         self.r.incr(month_key, num)
         self.r.incr(year_key, num)
 
+        if category:
+            self._categorize(slug, category)
+
     def get_metric(self, slug):
         """Get the current values for a metric.
 
@@ -163,6 +224,40 @@ class R(object):
             results.append((slug, self.get_metric(slug)))
         return results
 
+    def get_category_metrics(self, category):
+        """Get metrics belonging to the given category"""
+        slug_list = self._category_slugs(category)
+        return self.get_metrics(slug_list)
+
+    def delete_category(self, category):
+        """Removes the category from Redis. This doesn't touch the metrics;
+        they simply become uncategorized."""
+        # Remove mapping of metrics-to-category
+        category_key = self._category_key(category)
+        self.r.delete(category_key)
+
+        # Remove category from Set
+        self.r.srem(self._categories_key, category)
+
+    def reset_category(self, category, metric_slugs):
+        """Resets (or creates) a category containing a list of metrics.
+
+        * ``category`` -- A category name
+        * ``metric_slugs`` -- a list of all metrics that are members of the
+            category.
+
+        """
+        if len(metric_slugs) == 0:
+            # If there are no metrics, just remove the category
+            self.delete_category(category)
+        else:
+            # Convert metrics to json (removing any duplicates)
+            json_data = json.dumps(list(metric_slugs))
+            self.r.set(self._category_key(category), json_data)
+
+            # Store all category names in a Redis set, for easy retrieval
+            self.r.sadd(self._categories_key, category)
+
     def get_metric_history(self, slugs, since=None, granularity='daily'):
         """Get history for one or more metrics.
 
