@@ -6,6 +6,8 @@ the views here are extended from ``TemplateView`` in order to keep things
 simple.
 
 """
+from datetime import datetime
+
 from django.contrib.auth.decorators import user_passes_test
 from django.core.urlresolvers import reverse
 from django.http import Http404
@@ -14,7 +16,7 @@ from django.views.generic.edit import FormView
 from django.views.generic import TemplateView
 
 from .models import R
-from .forms import AggregateMetricForm
+from .forms import AggregateMetricForm, MetricCategoryForm
 
 
 class ProtectedTemplateView(TemplateView):
@@ -46,7 +48,9 @@ class MetricsListView(ProtectedTemplateView):
         """Includes the metrics slugs in the context."""
         data = super(MetricsListView, self).get_context_data(**kwargs)
         r = R()
-        data['metric_slugs'] = r.metric_slugs()
+        # Metrics organized by category, like so:
+        # { <category_name>: [ <slug1>, <slug2>, ... ]}
+        data['metrics'] = r.metric_slugs_by_category()
 
         # Include gauges in the default View. They're *technically* a list
         # of metrics, too!
@@ -79,11 +83,17 @@ class MetricHistoryView(ProtectedTemplateView):
         """Includes the metrics slugs in the context."""
         data = super(MetricHistoryView, self).get_context_data(**kwargs)
         try:
+            # Accept GET query params for ``since``
+            since = self.request.GET.get('since', None)
+            if since and len(since) == 10:  # yyyy-mm-dd
+                since = datetime.strptime(since, "%Y-%m-%d")
+
             data.update({
                 'slug': kwargs['slug'],
                 'granularity': kwargs['granularity'],
                 'metric_history': R().get_metric_history(
                     slugs=kwargs['slug'],
+                    since=since,
                     granularity=kwargs['granularity']
                 )
             })
@@ -138,14 +148,45 @@ class AggregateHistoryView(ProtectedTemplateView):
         slug_set = set(kwargs['slugs'].split('+'))
         granularity = kwargs.get('granularity', 'daily')
         try:
+            # Accept GET query params for ``since``
+            since = self.request.GET.get('since', None)
+            if since and len(since) == 10:  # yyyy-mm-dd
+                since = datetime.strptime(since, "%Y-%m-%d")
+
             data.update({
                 'slugs': slug_set,
                 'granularity': granularity,
                 'metric_history': R().get_metric_history_as_columns(
                     slugs=list(slug_set),
+                    since=since,
                     granularity=granularity
                 )
             })
         except KeyError:
             raise Http404
         return data
+
+
+class CategoryFormView(ProtectedFormView):
+    """Allows the user to (re)categorize existing metrics."""
+    template_name = "redis_metrics/categorize.html"
+    form_class = MetricCategoryForm
+
+    def get(self, *args, **kwargs):
+        """See if this view was called with a specified category."""
+        self.initial = {"category_name":  kwargs.get('category_name', None)}
+        return super(CategoryFormView, self).get(*args, **kwargs)
+
+    def get_initial(self):
+        """Initialize the form with the given category."""
+        return self.initial
+
+    def get_success_url(self):
+        """Show the list of metrics."""
+        return reverse('redis_metrics_list')
+
+    def form_valid(self, form):
+        """Get the category name/metric slugs from the form, and update the
+        category so contains the given metrics."""
+        form.categorize_metrics()
+        return super(CategoryFormView, self).form_valid(form)
